#!/usr/bin/perl -w
# 
# sb::Plugin::ColorfulCalendar - Plugin Module for Serene Bach
# - Written by Gin-Lime / published on 2006-05-29
# -- `Serene Bach' is written by T.Otani
# --   Copyright (C) 2005- SimpleBoxes/SerendipityNZ.

package sb::Plugin::ColorfulCalendar;
# ==================================================
# // initialization for plugin
# ==================================================
use sb::Plugin ();
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'euc',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin, cms',
		'name'    => 'Colorful Calendar',
		'text'    => 'Decorates a calendar colorfully',
		'author'  => 'Gin-Lime',
		'detail'  => 'http://www.emeraldshell.com/sb/',
		'version' => '0.03',
		'publish' => 'yes',	# ץ饰ꥹȤΤγĥʸ/
	},
	'file' => 'colcal.txt',
	'data' => 1,
);
# cmsѥץ饰ȤϿ
sb::Plugin->register_content_module(
	'type'     => 'main',
	'callback' => \&sb::Content::ColCal::_calendar,
	'field'    => 'calendar',
);
# adminѥץ饰ȤϿ
sb::Plugin->register_admin_module(
	'mode'   => 'colcal',
	'level'  => 2,
	'module' => 'sb::Admin::ColCal',
);
package sb::Content::ColCal;
use sb::Config ();
# ==================================================
# // functions for content
# ==================================================
use sb::Time ();
use sb::Language ();
use sb::Content ();
use sb::Text ();
sub _calendar { # 
	my $cms = shift;
	my %var = @_;
	my $set_data = sb::Plugin->get_data;
	my @settings = split('<>',$set_data->setting);
	my $output_table = ($settings[0] ne '') ? $settings[0] : &sb::Admin::ColCal::OUTPUT_TABLE;
	my $span_separator = ($settings[1] ne '') ? $settings[1] : &sb::Admin::ColCal::SPAN_SEPARATOR;
	my $table_spacing = ($settings[2] ne '') ? $settings[2] : &sb::Admin::ColCal::TABLE_SPACING;
	my @strings = split('<>',$set_data->text);
	my $empty_contents = ($strings[0] ne '') ? $strings[0] : &sb::Admin::ColCal::EMPTY_CONTENTS;
	$empty_contents = sb::Text->detitize($empty_contents);
	my $prev_arrow = ($strings[1] ne '') ? sb::Text->detitize($strings[1]) : &sb::Content::PREV_ARROW;
	my $next_arrow = ($strings[2] ne '') ? sb::Text->detitize($strings[2]) : &sb::Content::NEXT_ARROW;

	my %cal = ('no1'=>'','no2'=>'','hor'=>'','ver'=>'','tab'=>'');
	my $zone = $var{'conf'}->value('conf_timezone');
	my $date = ($var{'mode'} eq 'arc') 
		? substr($var{'cond'},0,6) 
		: sb::Time->format(
			'time'=>$var{'time'},
			'form'=>'%Year%%Mon%',
			'zone'=>$zone,
		);
	my $year  = substr($date,0,4);
	my $mon   = substr($date,4,2);
	my $week  = sb::Time->get_weekday('year'=>$year,'mon'=>$mon);
	my $end   = sb::Time->get_lastday('year'=>$year,'mon'=>$mon);
	my $today = sb::Time->format(
		'time'=>$var{'time'},
		'form'=>'%Year%%Mon%%Day%',
		'zone'=>$zone,
	);
	my $holidays = sb::Language->get->holiday($year);
	my $cgi   = $var{'conf'}->value('conf_srv_cgi') . $var{'conf'}->value('basic_sb') . '?day=';
	my $label = sb::Time->format(
		'time'=>($var{'mode'} eq 'arc')
			? sb::Time->convert('year'=>$year,'mon'=>$mon,'zone'=>$zone)
			: $var{'time'},
		'form'=>$var{'conf'}->value('conf_archivelist'),
		'lang'=>$var{'conf'}->value('conf_time_lang'),
		'zone'=>$zone,
	);
	my $lang = sb::Language->get();
	my @week_header_class = ();
	my @week_header = ();
	my @name_week = ();
	for(my $i=0;$i<7;$i++) {
		$week_header_class[$i] = lc($lang->string('week_enlong')->[$i]);
		$week_header[$i] = $lang->string('week_' . $var{'conf'}->value('conf_time_lang'))->[$i];
		$name_week[$i] = substr($week_header_class[$i], 0, 3);
	}
	my $prev  = ($mon != 1)  ? $year . &sb::Content::_pad0($mon - 1) : ($year - 1) . '12';
	my $next  = ($mon != 12) ? $year . &sb::Content::_pad0($mon + 1) : ($year + 1) . '01';
	$prev = '<a href="' . &sb::Content::_monthly_link($prev,$var{'conf'}) . '">' . $prev_arrow . '</a>';
	$next = '<a href="' . &sb::Content::_monthly_link($next,$var{'conf'}) . '">' . $next_arrow . '</a>';
	my $hs = ($span_separator) ? ' ' : '';
	{ # ǽν
		$cal{'no1'}  = '<table border="0" cellspacing="' . $table_spacing . '" cellpadding="0" class="calendar">' . "\n";
		$cal{'no2'}  = '<table border="0" cellspacing="' . $table_spacing . '" cellpadding="0" class="calendar">' . "\n";
		$cal{'no2'} .= '<tr>' . "\n" . '<td colspan="7" class="calendar_month">' . $prev . ' ';
		$cal{'no2'} .= $label . ' ' . $next . '</td>' . "\n" . '</tr>' . "\n";
		$cal{'no2'} .= '<tr>' . "\n";
		for(my $i=0;$i<7;$i++) { # إåν
			$cal{'no2'} .= '<td class="' . $week_header_class[$i] . '">' . $week_header[$i] . '</td>' . "\n";
		}
		$cal{'no2'} .= '</tr>' . "\n";
		if($output_table) {
			$cal{'ver'}  = '<table border="0" cellspacing="' . $table_spacing . '" cellpadding="0" class="calendar">' . "\n";
			$cal{'ver'} .= '<tr>' . "\n" . '<td class="calendar_month">' . $mon . '</td>' . "\n" . '</tr>' . "\n";
			$cal{'ver'} .= '<tr>' . "\n" . '<td class="calendar_separator">--</td>' . "\n" . '</tr>' . "\n";
			$cal{'hor'}  = '<table border="0" cellspacing="' . $table_spacing . '" cellpadding="0" class="calendar">' . "\n";
			$cal{'hor'} .= '<tr>' . "\n" . '<td class="calendar_prev">' . $prev . '</td>' . "\n";
			$cal{'hor'} .= '<td class="calendar_month">' . $label . '</td>' . "\n";
			$cal{'hor'} .= '<td class="calendar_separator">|</td>' . "\n";
		} else {
			$cal{'ver'}  = '<br />' . "\n" . '<span class="calendar_month">' . $mon;
			$cal{'ver'} .= '</span><br />' . "\n" . '<span class="calendar_separator">--</span><br />' . "\n";
			$cal{'hor'}  = '<span class="calendar_prev">' . $prev . '</span>';
			$cal{'hor'} .= $hs . '<span class="calendar_month">';
			$cal{'hor'} .= $label . '</span>' . $hs . '<span class="calendar_separator">|</span>' . $hs;
		}
	}
	my $times_week = 1;
	for (my $d=1;$d<=$end;$d++) { # 
		my $w = ($d - 1 + $week) % 7;
		my $check = $date . &sb::Content::_pad0($d);

		if ($d == 1 or ($w == 0 and $d > 1)) {
			$cal{'tab'} .= '<tr class="week_' . $times_week . '">' . "\n";
		}

		if ($d == 1 and $w > 0) {
			for(my $i=0;$i<$w;$i++) {
				$cal{'tab'} .= '<td class="cell_' . $name_week[$i] . '_empty">' . $empty_contents . '</td>' . "\n";
			}
		}

		my $this_date = $mon . &sb::Content::_pad0($d);
		my $date_name = $holidays->{$this_date};
		my $cell_class = $name_week[$w];
		if($date_name ne '') {
			$cell_class .=  ' holiday';
		}
		if($check eq $today) {
			$cell_class .=  ' today';
		}

		my $buf_cell = '<td class="cell_';
		my $buf = '<span class="span_';

		my $buf_common = $cell_class;
		$buf_common .= '" title="' . $date_name if($date_name ne '');
		$buf_common .= '">';

		$buf_cell .= $buf_common;
		$buf .= $buf_common;

		if ($var{'entryinfo'}{'daily'}{$check}) {
			$buf_cell .= '<a href="' . $cgi . $check . '">';
			$buf .= '<a href="' . $cgi . $check . '">';
		}
		{ # ɽ
			$buf_cell .= $d;
			$buf .= ($check eq $today) ? '<span class="today">' . $d . '</span>' : $d;
		}
		if ($var{'entryinfo'}{'daily'}{$check}) {
			$buf_cell .= '</a>';
			$buf .= '</a>';
		}
		$buf_cell .= '</td>' . "\n";
		$cal{'tab'} .= $buf_cell;

		if($output_table) {
			$cal{'ver'} .= '<tr>' . "\n" . $buf_cell . '</tr>' . "\n";
			$cal{'hor'} .= $buf_cell;
		} else {
			$cal{'hor'} .= $buf . '</span>' . $hs;
			$cal{'ver'} .= $buf . '</span><br />' . "\n";
		}
		if ($d == $end) {
			for(my $i=$w+1;$i<7;$i++) {
				$cal{'tab'} .= '<td class="cell_' . $name_week[$i] . '_empty">' . $empty_contents . '</td>' . "\n";
			}
		}
		if ($d == $end or $w == 6) {
			$cal{'tab'} .= '</tr>' . "\n";
			$times_week++;
		}
	}
	{ # Ǹν
		$cal{'no1'} .= $cal{'tab'} . '<tr><td colspan="7"><div style="text-align: center;" class="calendar_month">';
		$cal{'no1'} .= $prev . ' ' . $label . ' ' . $next . '</div></td></tr>' . "\n" . '</table>' . "\n";
		$cal{'no2'} .= $cal{'tab'} . '</table>';
		if($output_table) {
			$cal{'ver'} .= '<tr>' . "\n" . '<td class="calendar_separator">--</td>' . '</tr>' . "\n";
			$cal{'ver'} .= '<tr>' . "\n" . '<td class="calendar_next">' . $next . '</td>' . '</tr>' . "\n";
			$cal{'ver'} .= '<tr>' . "\n" . '<td class="calendar_prev">' . $prev . '</td>' . '</tr>' . "\n";
			$cal{'ver'} .= '<tr>' . "\n" . '<td class="calendar_separator">--</td>' . '</tr>' . "\n";
			$cal{'ver'} .= '</table>' . "\n";
			$cal{'hor'} .= '<td class="calendar_next">' . $next . '</td>' . "\n";
			$cal{'hor'} .= '</tr>' . "\n" . '</table>' . "\n";
		} else {
			$cal{'ver'} .= '<span class="calendar_separator">--</span><br />' . "\n" . '<span class="calendar_next">' . $next . '</span><br />' . "\n";
			$cal{'ver'} .= '<span class="calendar_prev">' . $prev . '</span><br />' . "\n" . '<span class="calendar_separator">--</span><br />' . "\n";
			$cal{'hor'} .= '<span class="calendar_next">' . $next . '</span>' . "\n";
		}
	}
	$cms->num(0);
	$cms->tag('calendar'=>$cal{'no1'});
	$cms->tag('calendar2'=>$cal{'no2'});
	$cms->tag('calendar_vertical'=>$cal{'ver'});
	$cms->tag('calendar_horizontal'=>$cal{'hor'});
	return(1);
}

package sb::Admin::ColCal;
use sb::Text ();
use vars qw( @ISA );
use sb::Interface ();
use sb::TemplateManager ();
@ISA = qw( sb::App::Admin );
# ==================================================
# // declaration for constant value
# ==================================================
sub TEMPLATE  (){ 'colcal.html' };
# ķơ֥ǽϤ0:̾/1:ơ֥
sub OUTPUT_TABLE (){ '0' };
# ǥơ֥Ϥʤˡդդδ֤˥ڡ뤫ɤ
#0:ʤ/1:
sub SPAN_SEPARATOR (){ '1' };
# ơ֥륻Υڡ󥰡ʿͤǻ
sub TABLE_SPACING (){ '0' };
# Υơʲʤɤ֤ġ
sub EMPTY_CONTENTS (){ '&amp;nbsp;' };
# ؤΥ󥯤
sub PREV_ARROW_DEFAULT (){ '' };
# ؤΥ󥯤
sub NEXT_ARROW_DEFAULT (){ '' };
# ==================================================
# // public functions - callback
# ==================================================
sub callback {
	my $self = shift;
	return ( $self->{'regi'} ) 
		? $self->_save_setting(@_)
		: $self->_show_setting(@_);
}
# ==================================================
# // private functions - main routine
# ==================================================
sub _save_setting {
	my $self = shift;
	my $cgi  = sb::Interface->get;
	# ǡμ
	my $data = sb::Plugin->get_data;
	# μ
	my $span_separator = ($cgi->value('span_separator') ne '') ? $cgi->value('span_separator') : $cgi->value('span_separator_hidden');
	$data->setting($cgi->value('output_table') . '<>' . $span_separator . '<>' . $cgi->value('table_spacing'));
	$data->text(sb::Text->entitize($cgi->value('empty_contents')) . '<>' . sb::Text->entitize($cgi->value('prev_arrow')) . '<>' . sb::Text->entitize($cgi->value('next_arrow')));
	# ǡ¸
	sb::Plugin->set_data('data' => $data);
	# ¸λåɽ
	my $lang = sb::Language->get;
	my $msg = $lang->string('parts_confcomp');
	$msg .= $lang->string('parts_rec_make');
	$msg .= sprintf($lang->string('parts_link_bld'),&sb::App::Admin::get_script_path);
	return $self->_show_setting('message'=>$msg);
}
sub _show_setting {
	my $self = shift;
	my %param = ('message' => '', @_);
	my $cms  = sb::TemplateManager->new(sb::Plugin->load_template('file' => TEMPLATE));
	my $set_data = sb::Plugin->get_data;
	my @settings = ();
	@settings = split('<>',$set_data->setting) if($set_data ne '');
	my $output_table = ($settings[0] ne '') ? $settings[0] : OUTPUT_TABLE;
	my $span_separator = ($settings[1] ne '') ? $settings[1] : SPAN_SEPARATOR;
	my $table_spacing = ($settings[2] ne '') ? $settings[2] : TABLE_SPACING;
	my @strings = ();
	@strings = split('<>',$set_data->text) if($set_data ne '');
	my $empty_contents = ($strings[0] ne '') ? $strings[0] : EMPTY_CONTENTS;
	my $prev_arrow = ($strings[1] ne '') ? $strings[1] : '';
	my $next_arrow = ($strings[2] ne '') ? $strings[2] : '';

	# ȼͤ
	$cms->num(0);
	$cms->tag('sb_colcal_output_table_0' =>
	          (!$output_table) ? ' selected' : '' );
	$cms->tag('sb_colcal_output_table_1' =>
	          ($output_table) ? ' selected' : '' );
	$cms->tag('sb_colcal_span_separator_0' =>
	          (!$span_separator) ? ' selected' : '' );
	$cms->tag('sb_colcal_span_separator_1' =>
	          ($span_separator) ? ' selected' : '' );
	$cms->tag('sb_colcal_span_separator_hidden' => $span_separator);
	$cms->tag('sb_colcal_table_spacing' => $table_spacing);
	$cms->tag('sb_colcal_empty_contents' => $empty_contents);
	$cms->tag('sb_colcal_prev_arrow' => $prev_arrow);
	$cms->tag('sb_colcal_next_arrow' => $next_arrow);

	# åɽ
	if ($param{'message'} ne '') {
		$cms->num(0);
		$cms->tag('sb_process_message' => $param{'message'});
		$cms->block('sb_process_message' => 1);
	}

	# ̤ɽ
	return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}

1;
# require 뤿Τޤʤ
__END__

